import { txClient, queryClient } from './module'
// @ts-ignore
import { SpVuexError } from '@starport/vuex'
import { Coin } from './module/types/cosmos/base/v1beta1/coin'
import { DecCoin } from './module/types/cosmos/base/v1beta1/coin'
import { IntProto } from './module/types/cosmos/base/v1beta1/coin'
import { DecProto } from './module/types/cosmos/base/v1beta1/coin'
export { Coin, DecCoin, IntProto, DecProto }
async function initTxClient(vuexGetters) {
	return await txClient(vuexGetters['common/wallet/signer'], {
		addr: vuexGetters['common/env/apiTendermint']
	})
}
async function initQueryClient(vuexGetters) {
	return await queryClient({
		addr: vuexGetters['common/env/apiCosmos']
	})
}
function mergeResults(value, next_values) {
	for (let prop of Object.keys(next_values)) {
		if (Array.isArray(next_values[prop])) {
			value[prop] = [...value[prop], ...next_values[prop]]
		} else {
			value[prop] = next_values[prop]
		}
	}
	return value
}
function getStructure(template) {
	let structure = { fields: [] }
	for (const [key, value] of Object.entries(template)) {
		let field = {}
		field.name = key
		field.type = typeof value
		structure.fields.push(field)
	}
	return structure
}
const getDefaultState = () => {
	return {
		_Structure: {
			Coin: getStructure(Coin.fromPartial({})),
			DecCoin: getStructure(DecCoin.fromPartial({})),
			IntProto: getStructure(IntProto.fromPartial({})),
			DecProto: getStructure(DecProto.fromPartial({}))
		},
		_Subscriptions: new Set()
	}
}
// initial state
const state = getDefaultState()
export default {
	namespaced: true,
	state,
	mutations: {
		RESET_STATE(state) {
			Object.assign(state, getDefaultState())
		},
		QUERY(state, { query, key, value }) {
			state[query][JSON.stringify(key)] = value
		},
		SUBSCRIBE(state, subscription) {
			state._Subscriptions.add(subscription)
		},
		UNSUBSCRIBE(state, subscription) {
			state._Subscriptions.delete(subscription)
		}
	},
	getters: {
		getTypeStructure: (state) => (type) => {
			return state._Structure[type].fields
		}
	},
	actions: {
		init({ dispatch, rootGetters }) {
			console.log('Vuex module: cosmos.base.v1beta1 initialized!')
			if (rootGetters['common/env/client']) {
				rootGetters['common/env/client'].on('newblock', () => {
					dispatch('StoreUpdate')
				})
			}
		},
		resetState({ commit }) {
			commit('RESET_STATE')
		},
		unsubscribe({ commit }, subscription) {
			commit('UNSUBSCRIBE', subscription)
		},
		async StoreUpdate({ state, dispatch }) {
			state._Subscriptions.forEach(async (subscription) => {
				try {
					await dispatch(subscription.action, subscription.payload)
				} catch (e) {
					throw new SpVuexError('Subscriptions: ' + e.message)
				}
			})
		}
	}
}
